import {
  renderMonthGridView
} from "./chunk.RB5IEYTY.js";
import {
  TransitionalStyles
} from "./chunk.FRKST3V3.js";
import {
  tokens
} from "./chunk.LOUXU5LA.js";
import {
  watch
} from "./chunk.2WO4NHJ2.js";
import {
  GdsElement
} from "./chunk.ZQ4D5K7J.js";
import {
  gdsCustomElement
} from "./chunk.VOYMQ322.js";
import {
  __decorateClass,
  __privateAdd,
  __privateMethod,
  __spreadValues
} from "./chunk.5VURDMKE.js";

// libs/core/src/components/calendar/calendar.ts
import { html } from "lit";
import { classMap } from "lit/directives/class-map.js";
import { ifDefined } from "lit/directives/if-defined.js";
import { when } from "lit/directives/when.js";
import { property, query } from "lit/decorators.js";
import { msg } from "@lit/localize";
import {
  addDays,
  isSameDay,
  isSameMonth,
  getWeek,
  subMonths,
  addMonths,
  lastDayOfMonth
} from "date-fns";

// libs/core/src/components/calendar/calendar.styles.ts
import { css } from "lit";
var style = css`
  @layer base, reset;

  @layer base {
    :host {
      display: flex;
      justify-content: center;
    }

    :host table {
      border-spacing: var(--gds-space-2xs);
      flex-grow: 1;
      width: 100%;
      padding: 1ch;
    }

    thead {
      box-shadow: 0 var(--gds-space-4xs) 0 0 var(--gds-sys-color-base300);
      th {
        height: var(--gds-space-2xl);
        width: var(--gds-space-2xl);
        background: var(--gds-sys-color-container-container-bright);
        box-sizing: border-box;
        text-align: center;
        font-weight: normal;
      }
    }

    tbody {
      td {
        position: relative;
        height: var(--gds-space-2xl);
        width: var(--gds-space-2xl);
        box-sizing: border-box;
        text-align: center;
        user-select: none;
        border-width: var(--gds-space-3xs);
        border-style: solid;
        border-color: transparent;
        border-radius: var(--gds-space-2xs);
        transition: all 0.2s;
        outline-style: solid;
        outline-color: transparent;
        outline-width: 0px;
        outline-offset: var(--gds-space-3xs);

        &:not(.disabled):hover,
        &[aria-selected='true'] {
          background: var(--gds-sys-color-base800);
          cursor: pointer;
          color: var(--gds-sys-color-base-white);
        }

        &.today {
          border-color: var(--gds-sys-color-base800);
        }

        &.disabled {
          background-color: var(--gds-sys-color-base100);
          color: var(--gds-sys-color-base500);
          cursor: not-allowed;
        }

        &:active:not(.disabled) {
          scale: 0.96;
        }

        &:focus-visible {
          outline-color: color-mix(in srgb, currentColor, transparent 50%);
        }
      }

      .indicator-dot {
        display: flex;
        position: absolute;
        width: var(--gds-space-2xs);
        height: var(--gds-space-2xs);
        border-radius: var(--gds-space-max);
        background-color: var(--_color, currentColor);
        inset: var(--gds-space-2xs) var(--gds-space-2xs) auto auto;
      }
    }
  }
`;
var calendar_styles_default = style;

// libs/core/src/components/calendar/calendar.ts
var _setSelectedDate, setSelectedDate_fn, _handleKeyDown, handleKeyDown_fn;
var GdsCalendar = class extends GdsElement {
  constructor() {
    super(...arguments);
    __privateAdd(this, _setSelectedDate);
    __privateAdd(this, _handleKeyDown);
    this.min = new Date((/* @__PURE__ */ new Date()).getFullYear() - 10, 0, 1);
    this.max = new Date((/* @__PURE__ */ new Date()).getFullYear() + 10, 0, 1);
    this.focusedDate = /* @__PURE__ */ new Date();
    this.disabledWeekends = false;
    this.showWeekNumbers = false;
    this.hideExtraneousDays = false;
    this.hideDayNames = false;
    this.dateLabelTemplate = (date) => date.toDateString();
  }
  get focusedMonth() {
    return this.focusedDate.getMonth();
  }
  set focusedMonth(month) {
    const lastOfSelectedMonth = lastDayOfMonth(
      new Date(this.focusedYear, month, 1)
    );
    const newFocusedDate = new Date(this.focusedDate);
    newFocusedDate.setDate(
      Math.min(this.focusedDate.getDate(), lastOfSelectedMonth.getDate())
    );
    newFocusedDate.setMonth(month);
    newFocusedDate.setHours(12, 0, 0, 0);
    this.focusedDate = newFocusedDate;
  }
  get focusedYear() {
    return this.focusedDate.getFullYear();
  }
  set focusedYear(year) {
    this.focusedDate = new Date(this.focusedDate.setFullYear(year));
  }
  /**
   * Returns the date cell element for the given day number.
   */
  getDateCell(dayNumber) {
    var _a;
    return (_a = this.shadowRoot) == null ? void 0 : _a.querySelector(`#dateCell-${dayNumber}`);
  }
  connectedCallback() {
    super.connectedCallback();
    TransitionalStyles.instance.apply(this, "gds-calendar");
    this.addEventListener("keydown", __privateMethod(this, _handleKeyDown, handleKeyDown_fn));
  }
  focus() {
    var _a;
    super.focus();
    (_a = this._elFocusedCell) == null ? void 0 : _a.focus();
  }
  render() {
    const currentDate = /* @__PURE__ */ new Date();
    return html`<table role="grid" aria-label="${ifDefined(this.label)}">
      ${when(
      !this.hideDayNames,
      () => html`<thead role="rowgroup">
            <tr role="row">
              ${when(this.showWeekNumbers, () => html`<th></th>`)}
              <th>${msg("Mon")}</th>
              <th>${msg("Tue")}</th>
              <th>${msg("Wed")}</th>
              <th>${msg("Thu")}</th>
              <th>${msg("Fri")}</th>
              <th>${msg("Sat")}</th>
              <th>${msg("Sun")}</th>
            </tr>
          </thead>`
    )}
      <tbody role="rowgroup">
        ${renderMonthGridView(
      this.focusedDate,
      (weeks) => html`
            ${weeks.map(
        (week) => html`
                <tr role="row">
                  ${when(
          this.showWeekNumbers,
          () => html`<td class="week-number" scope="row">
                        ${getWeek(week.days[0])}
                      </td>`
        )}
                  ${week.days.map((day) => {
          const customization = this.customizedDates && this.customizedDates.find((d) => isSameDay(d.date, day));
          const displayOptions = __spreadValues({
            // Defaults
            color: "currentColor",
            // Get baseline disabled state from the disabledDates prop (or false if unset)
            disabled: Boolean(
              this.disabledDates && this.disabledDates.some((d) => isSameDay(d, day))
            )
          }, customization);
          const isOutsideCurrentMonth = !isSameMonth(this.focusedDate, day) || day < this.min || day > this.max;
          const isWeekend = day.getDay() === 0 || day.getDay() === 6;
          const isDisabled = displayOptions.disabled || isOutsideCurrentMonth || this.disabledWeekends && isWeekend;
          const shouldRenderBlank = this.hideExtraneousDays && isOutsideCurrentMonth;
          return shouldRenderBlank ? html`<td inert></td>` : html`
                          <td
                            role="${ifDefined(
            isDisabled ? void 0 : "gridcell"
          )}"
                            class="${classMap({
            "custom-date": Boolean(customization),
            disabled: Boolean(isDisabled),
            today: isSameDay(currentDate, day)
          })}"
                            ?disabled=${isDisabled}
                            tabindex="${isSameDay(this.focusedDate, day) ? 0 : -1}"
                            aria-selected="${this.value && isSameDay(this.value, day) ? "true" : "false"}"
                            aria-label="${this.dateLabelTemplate(day)}"
                            @click=${() => isDisabled ? null : __privateMethod(this, _setSelectedDate, setSelectedDate_fn).call(this, day)}
                            id="dateCell-${day.getDate()}"
                          >
                            <span
                              class="number"
                              style="--_color: ${displayOptions ? displayOptions == null ? void 0 : displayOptions.color : ""}"
                              >${day.getDate()}</span
                            >

                            ${when(
            displayOptions.indicator,
            () => html`<span
                                  class="indicator-${displayOptions == null ? void 0 : displayOptions.indicator}"
                                  style="--_color: ${displayOptions == null ? void 0 : displayOptions.color}"
                                ></span>`
          )}
                          </td>
                        `;
        })}
                </tr>
              `
      )}
          `
    )}
      </tbody>
    </table>`;
  }
  _valueChanged() {
    if (!this.value)
      return;
    this.focusedDate = this.value;
  }
};
_setSelectedDate = new WeakSet();
setSelectedDate_fn = function(date) {
  this.value = date;
  this.dispatchEvent(
    new CustomEvent("change", {
      detail: date,
      bubbles: false,
      composed: false
    })
  );
};
_handleKeyDown = new WeakSet();
handleKeyDown_fn = function(e) {
  var _a;
  let handled = false;
  let newFocusedDate = new Date(this.focusedDate);
  if (e.key === "ArrowLeft") {
    newFocusedDate = addDays(this.focusedDate, -1);
    handled = true;
  } else if (e.key === "ArrowRight") {
    newFocusedDate = addDays(this.focusedDate, 1);
    handled = true;
  } else if (e.key === "ArrowUp") {
    newFocusedDate = addDays(this.focusedDate, -7);
    handled = true;
  } else if (e.key === "ArrowDown") {
    newFocusedDate = addDays(this.focusedDate, 7);
    handled = true;
  } else if (e.key === "Enter" || e.key === " ") {
    if (!((_a = this._elFocusedCell) == null ? void 0 : _a.hasAttribute("disabled"))) {
      __privateMethod(this, _setSelectedDate, setSelectedDate_fn).call(this, this.focusedDate);
    }
    handled = true;
  } else if (e.key === "Home") {
    newFocusedDate = new Date(this.focusedYear, this.focusedMonth, 1);
    handled = true;
  } else if (e.key === "End") {
    newFocusedDate = new Date(this.focusedYear, this.focusedMonth + 1, 0);
    handled = true;
  } else if (e.key === "PageUp") {
    newFocusedDate = subMonths(this.focusedDate, 1);
    handled = true;
  } else if (e.key === "PageDown") {
    newFocusedDate = addMonths(this.focusedDate, 1);
    handled = true;
  }
  if (newFocusedDate.getFullYear() >= this.min.getFullYear() && newFocusedDate.getFullYear() <= this.max.getFullYear()) {
    const proceed = this.dispatchEvent(
      new CustomEvent("gds-date-focused", {
        detail: newFocusedDate,
        bubbles: false,
        composed: false,
        cancelable: true
      })
    );
    if (proceed) {
      this.focusedDate = newFocusedDate;
    }
  }
  if (handled) {
    e.preventDefault();
    e.stopPropagation();
    this.updateComplete.then(() => {
      var _a2;
      (_a2 = this._elFocusedCell) == null ? void 0 : _a2.focus();
    });
  }
};
GdsCalendar.styles = [tokens, calendar_styles_default];
GdsCalendar.shadowRootOptions = {
  mode: "open",
  delegatesFocus: true
};
__decorateClass([
  property()
], GdsCalendar.prototype, "value", 2);
__decorateClass([
  property({ type: Date })
], GdsCalendar.prototype, "min", 2);
__decorateClass([
  property({ type: Date })
], GdsCalendar.prototype, "max", 2);
__decorateClass([
  property()
], GdsCalendar.prototype, "focusedDate", 2);
__decorateClass([
  property({ type: Boolean, attribute: "disabled-weekends" })
], GdsCalendar.prototype, "disabledWeekends", 2);
__decorateClass([
  property({ type: Array, attribute: "disabled-dates" })
], GdsCalendar.prototype, "disabledDates", 2);
__decorateClass([
  property({ type: Number })
], GdsCalendar.prototype, "focusedMonth", 1);
__decorateClass([
  property({ type: Number })
], GdsCalendar.prototype, "focusedYear", 1);
__decorateClass([
  property({ type: Boolean })
], GdsCalendar.prototype, "showWeekNumbers", 2);
__decorateClass([
  property({ type: Boolean })
], GdsCalendar.prototype, "hideExtraneousDays", 2);
__decorateClass([
  property({ type: Boolean })
], GdsCalendar.prototype, "hideDayNames", 2);
__decorateClass([
  property({ attribute: false })
], GdsCalendar.prototype, "customizedDates", 2);
__decorateClass([
  property()
], GdsCalendar.prototype, "label", 2);
__decorateClass([
  property({ attribute: false })
], GdsCalendar.prototype, "dateLabelTemplate", 2);
__decorateClass([
  query('td[tabindex="0"]')
], GdsCalendar.prototype, "_elFocusedCell", 2);
__decorateClass([
  watch("value")
], GdsCalendar.prototype, "_valueChanged", 1);
GdsCalendar = __decorateClass([
  gdsCustomElement("gds-calendar")
], GdsCalendar);

export {
  GdsCalendar
};
