import { Placement } from '@floating-ui/dom';
import { GdsElement } from '../../gds-element';
import '../../primitives/menu';
import '../popover';
/**
 * @element gds-context-menu
 * @status beta
 *
 * The context menu is presented in-context to the related data, so users don’t lose
 * their place in the app. It appears upon interaction with the 3 dot button and
 * displays a list of choices, one choice per line.
 *
 * @event gds-menu-item-click - Fired when a menu item is clicked.
 * @event gds-ui-state - Fired when the menu is opened or closed.
 *
 * @slot - The default slot. Only `gds-menu-item` elements are accepted here.
 */
export declare class GdsContextMenu extends GdsElement {
    #private;
    static styles: import("lit").CSSResult[];
    static shadowRootOptions: ShadowRootInit;
    /**
     * Whether the context menu popover is open.
     */
    open: boolean;
    /**
     * The label for the trigger button.
     */
    buttonLabel: string;
    /**
     * Whether to show the label on the trigger button.
     */
    showLabel: boolean;
    /**
     * The label for the popover and menu. Should describe the context of the menu.
     */
    label: string;
    /**
     * The placement of the popover relative to the trigger.
     */
    placement: Placement;
    private elTriggerBtn;
    constructor();
    connectedCallback(): void;
    render(): any;
}
